@@ -145,10 +145,13 @@
 #include "lib/activate/activate.h"
 #include "lib/label/hints.h"
 #include "lib/device/dev-type.h"
+#include "lib/device/device_id.h"
+#include "lib/device/online.h"
 
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <dirent.h>
 #include <time.h>
 #include <sys/types.h>
 #include <sys/file.h>
@@ -166,8 +169,10 @@ static const char *_newhints_file = DEFAULT_RUN_DIR "/newhints";
  * than they were built with.  Increase the minor number
  * when adding features that older lvm versions can just
  * ignore while continuing to use the other content.
+ *
+ * MAJOR 2: add devices_file
  */
-#define HINTS_VERSION_MAJOR 1
+#define HINTS_VERSION_MAJOR 2
 #define HINTS_VERSION_MINOR 1
 
 #define HINT_LINE_LEN (PATH_MAX + NAME_LEN + ID_LEN + 64)
@@ -190,8 +195,10 @@ static int _hints_exists(void)
 
 	if (!stat(_hints_file, &buf))
 		return 1;
+
 	if (errno != ENOENT)
-		log_debug("hints_exist errno %d", errno);
+		log_debug("hints_exist errno %d %s", errno, _hints_file);
+
 	return 0;
 }
 
@@ -201,8 +208,10 @@ static int _nohints_exists(void)
 
 	if (!stat(_nohints_file, &buf))
 		return 1;
+
 	if (errno != ENOENT)
-		log_debug("nohints_exist errno %d", errno);
+		log_debug("nohints_exist errno %d %s", errno, _nohints_file);
+
 	return 0;
 }
 
@@ -212,8 +221,10 @@ static int _newhints_exists(void)
 
 	if (!stat(_newhints_file, &buf))
 		return 1;
+
 	if (errno != ENOENT)
-		log_debug("newhints_exist errno %d", errno);
+		log_debug("newhints_exist errno %d %s", errno, _newhints_file);
+
 	return 0;
 }
 
@@ -225,6 +236,7 @@ static int _touch_newhints(void)
 		return_0;
 	if (fclose(fp))
 		stack;
+	log_debug("newhints created");
 	return 1;
 }
 
@@ -243,29 +255,33 @@ static int _touch_hints(void)
 {
 	FILE *fp;
 
-	if (!(fp = fopen(_hints_file, "w")))
-		return_0;
+	if (!(fp = fopen(_hints_file, "w"))) {
+		log_debug("touch_hints errno %d %s", errno, _hints_file);
+		return 0;
+	}
 	if (fclose(fp))
-		stack;
+		log_debug("touch_hints close errno %d %s", errno, _hints_file);
+
 	return 1;
 }
 
 static void _unlink_nohints(void)
 {
 	if (unlink(_nohints_file))
-		log_debug("unlink_nohints errno %d", errno);
+		log_debug("unlink_nohints errno %d %s", errno, _nohints_file);
 }
 
+
 static void _unlink_hints(void)
 {
 	if (unlink(_hints_file))
-		log_debug("unlink_hints errno %d", errno);
+		log_debug("unlink_hints errno %d %s", errno, _hints_file);
 }
 
 static void _unlink_newhints(void)
 {
 	if (unlink(_newhints_file))
-		log_debug("unlink_newhints errno %d", errno);
+		log_debug("unlink_newhints errno %d %s", errno, _newhints_file);
 }
 
 static int _clear_hints(struct cmd_context *cmd)
@@ -274,7 +290,7 @@ static int _clear_hints(struct cmd_context *cmd)
 	time_t t;
 
 	if (!(fp = fopen(_hints_file, "w"))) {
-		log_warn("Failed to clear hint file.");
+		log_debug("clear_hints open errno %d", errno);
 		/* shouldn't happen, but try to unlink in case */
 		_unlink_hints();
 		return 0;
@@ -285,10 +301,10 @@ static int _clear_hints(struct cmd_context *cmd)
 	fprintf(fp, "# Created empty by %s pid %d %s", cmd->name, getpid(), ctime(&t));
 
 	if (fflush(fp))
-		log_debug("clear_hints flush errno %d", errno);
+		log_debug("clear_hints flush errno %d %s", errno, _hints_file);
 
 	if (fclose(fp))
-		log_debug("clear_hints close errno %d", errno);
+		log_debug("clear_hints close errno %d %s", errno, _hints_file);
 
 	return 1;
 }
@@ -312,7 +328,7 @@ static int _lock_hints(struct cmd_context *cmd, int mode, int nonblock)
 
 	fd = open(_hints_file, O_RDWR);
 	if (fd < 0) {
-		log_debug("lock_hints open errno %d", errno);
+		log_debug("lock_hints open errno %d %s", errno, _hints_file);
 		return 0;
 	}
 
@@ -324,7 +340,8 @@ static int _lock_hints(struct cmd_context *cmd, int mode, int nonblock)
 	}
 
 	if (close(fd))
-		stack;
+		log_debug("lock_hints close errno %d %s", errno, _hints_file);
+
 	return 0;
 }
 
@@ -351,10 +368,9 @@ static void _unlock_hints(struct cmd_context *cmd)
 
 void hints_exit(struct cmd_context *cmd)
 {
-	free_hints(&cmd->hints);
 	if (_hints_fd == -1)
 		return;
-	return _unlock_hints(cmd);
+	_unlock_hints(cmd);
 }
 
 void free_hints(struct dm_list *hints)
@@ -484,6 +500,8 @@ int validate_hints(struct cmd_context *cmd, struct dm_list *hints)
 	if (!(iter = dev_iter_create(NULL, 0)))
 		return 0;
 	while ((dev = dev_iter_get(cmd, iter))) {
+		if (dm_list_empty(&dev->aliases))
+			continue;
 		if (!(hint = _find_hint_name(hints, dev_name(dev))))
 			continue;
 
@@ -491,6 +509,19 @@ int validate_hints(struct cmd_context *cmd, struct dm_list *hints)
 		if (!hint->chosen)
 			continue;
 
+		/* 
+		 * label_scan was unable to read the dev so we don't know its pvid.
+		 * Since we are unable to verify the hint is correct, it's possible
+		 * that the PVID is actually found on a different device, so don't
+		 * depend on hints. (This would also fail the following pvid check.)
+		 */
+		if (dev->flags & DEV_SCAN_NOT_READ) {
+			log_debug("Uncertain hint for unread device %d:%d %s",
+				  major(hint->devt), minor(hint->devt), dev_name(dev));
+			ret = 0;
+			continue;
+		}
+
 		if (strcmp(dev->pvid, hint->pvid)) {
 			log_debug("Invalid hint device %d:%d %s pvid %s had hint pvid %s",
 				  major(hint->devt), minor(hint->devt), dev_name(dev),
@@ -663,7 +694,7 @@ static int _read_hint_file(struct cmd_context *cmd, struct dm_list *hints, int *
 	int found = 0;
 	int keylen;
 	int hv_major, hv_minor;
-	int major, minor;
+	int major = -1, minor = -1;
 	int ret = 1;
 	int i;
 
@@ -701,8 +732,9 @@ static int _read_hint_file(struct cmd_context *cmd, struct dm_list *hints, int *
 				break;
 			}
 
-			if (hv_major > HINTS_VERSION_MAJOR) {
-				log_debug("ignore hints with newer major version %d.%d", hv_major, hv_minor);
+			if (hv_major != HINTS_VERSION_MAJOR) {
+				log_debug("ignore hints with version %d.%d current %d.%d",
+					  hv_major, hv_minor, HINTS_VERSION_MAJOR, HINTS_VERSION_MINOR);
 				*needs_refresh = 1;
 				break;
 			}
@@ -737,7 +769,7 @@ static int _read_hint_file(struct cmd_context *cmd, struct dm_list *hints, int *
 
 		keylen = strlen("scan_lvs:");
 		if (!strncmp(_hint_line, "scan_lvs:", keylen)) {
-			int scan_lvs = 0;
+			unsigned scan_lvs = 0;
 			if ((sscanf(_hint_line + keylen, "%u", &scan_lvs) != 1) ||
 			    scan_lvs != cmd->scan_lvs) {
 				log_debug("ignore hints with different or unreadable scan_lvs");
@@ -747,6 +779,25 @@ static int _read_hint_file(struct cmd_context *cmd, struct dm_list *hints, int *
 			continue;
 		}
 
+		keylen = strlen("devices_file:");
+		if (!strncmp(_hint_line, "devices_file:", keylen)) {
+			const char *df_hint = _hint_line + keylen;
+			const char *df_config = find_config_tree_str(cmd, devices_devicesfile_CFG, NULL);
+			/* when a devices file is not used, hints should have devices_file:. */
+			if (!cmd->enable_devices_file || !df_hint || !df_config) {
+				if (df_hint[0] != '.') {
+					log_debug("ignore hints with different devices_file: not enabled vs %s", df_hint);
+					*needs_refresh = 1;
+					break;
+				}
+			} else if (strcmp(df_hint, df_config)) {
+				log_debug("ignore hints with different devices_file: %s vs %s", df_hint, df_config);
+				*needs_refresh = 1;
+				break;
+			}
+			continue;
+		}
+
 		keylen = strlen("devs_hash:");
 		if (!strncmp(_hint_line, "devs_hash:", keylen)) {
 			if (sscanf(_hint_line + keylen, "%u %u", &read_hash, &read_count) != 2) {
@@ -787,7 +838,7 @@ static int _read_hint_file(struct cmd_context *cmd, struct dm_list *hints, int *
 			if (!dm_strncpy(hint.vgname, vgname + 3, sizeof(hint.vgname)))
 				continue;
 
-		if (!(alloc_hint = malloc(sizeof(struct hint)))) {
+		if (!(alloc_hint = zalloc(sizeof(struct hint)))) {
 			ret = 0;
 			break;
 		}
@@ -799,7 +850,7 @@ static int _read_hint_file(struct cmd_context *cmd, struct dm_list *hints, int *
 	}
 
 	if (fclose(fp))
-		stack;
+		log_debug("read_hint_file close errno %d", errno);
 
 	if (!ret)
 		return 0;
@@ -816,10 +867,13 @@ static int _read_hint_file(struct cmd_context *cmd, struct dm_list *hints, int *
 	if (!(iter = dev_iter_create(NULL, 0)))
 		return 0;
 	while ((dev = dev_iter_get(cmd, iter))) {
+		if (cmd->enable_devices_file && !get_du_for_dev(cmd, dev))
+			continue;
+
 		if (!_dev_in_hint_hash(cmd, dev))
 			continue;
-		memset(devpath, 0, sizeof(devpath));
-		strncpy(devpath, dev_name(dev), PATH_MAX);
+
+		(void) dm_strncpy(devpath, dev_name(dev), sizeof(devpath));
 		calc_hash = calc_crc(calc_hash, (const uint8_t *)devpath, strlen(devpath));
 		calc_count++;
 	}
@@ -877,6 +931,7 @@ int write_hint_file(struct cmd_context *cmd, int newhints)
 	struct device *dev;
 	const char *vgname;
 	char *filter_str = NULL;
+	const char *config_devices_file = NULL;
 	uint32_t hash = INITIAL_CRC;
 	uint32_t count = 0;
 	time_t t;
@@ -937,6 +992,19 @@ int write_hint_file(struct cmd_context *cmd, int newhints)
 
 	fprintf(fp, "scan_lvs:%d\n", cmd->scan_lvs);
 
+	/*
+	 * Only associate hints with the default/system devices file.
+	 * If no default/system devices file is used, "." is set.
+	 * If we are using a devices file other than the config setting
+	 * (from --devicesfile), then we should not be using hints and
+	 * shouldn't get here.
+	 */
+	config_devices_file = find_config_tree_str(cmd, devices_devicesfile_CFG, NULL);
+	if (cmd->enable_devices_file && !cmd->devicesfile && config_devices_file)
+		fprintf(fp, "devices_file:%s\n", config_devices_file);
+	else
+		fprintf(fp, "devices_file:.\n");
+
 	/* 
 	 * iterate through all devs and write a line for each
 	 * dev flagged DEV_SCAN_FOUND_LABEL
@@ -954,6 +1022,9 @@ int write_hint_file(struct cmd_context *cmd, int newhints)
 	 * 2. add PVs to the hint file
 	 */
 	while ((dev = dev_iter_get(cmd, iter))) {
+		if (cmd->enable_devices_file && !get_du_for_dev(cmd, dev))
+			continue;
+
 		if (!_dev_in_hint_hash(cmd, dev)) {
 			if (dev->flags & DEV_SCAN_FOUND_LABEL) {
 				/* should never happen */
@@ -967,7 +1038,7 @@ int write_hint_file(struct cmd_context *cmd, int newhints)
 		 * detect when the devices on the system change, which
 		 * invalidates the existing hints.
 		 */
-		strncpy(devpath, dev_name(dev), PATH_MAX);
+		(void) dm_strncpy(devpath, dev_name(dev), sizeof(devpath));
 		hash = calc_crc(hash, (const uint8_t *)devpath, strlen(devpath));
 		count++;
 
@@ -1021,7 +1092,7 @@ int write_hint_file(struct cmd_context *cmd, int newhints)
 
  out_close:
 	if (fclose(fp))
-		stack;
+		log_debug("write_hint_file close errno %d", errno);
 
  out_unlock:
 	/* get_hints() took ex lock before returning with newhints set */
@@ -1119,8 +1190,10 @@ void pvscan_recreate_hints_begin(struct cmd_context *cmd)
 
 	log_debug("pvscan_recreate_hints_begin");
 
-	if (!_touch_hints())
+	if (!_touch_hints()) {
+		stack;
 		return;
+	}
 
 	/* limit potential delay blocking on hints lock next */
 	if (!_touch_nohints())
@@ -1157,8 +1230,8 @@ void invalidate_hints(struct cmd_context *cmd)
  * probably want to exclude that command from attempting this optimization,
  * because it would be difficult to know what VG that command wanted to use.
  */
-static void _get_single_vgname_cmd_arg(struct cmd_context *cmd,
-				       struct dm_list *hints, char **vgname)
+void get_single_vgname_cmd_arg(struct cmd_context *cmd,
+			       struct dm_list *hints, char **vgname)
 {
 	struct hint *hint;
 	char namebuf[NAME_LEN];
@@ -1207,6 +1280,11 @@ static void _get_single_vgname_cmd_arg(struct cmd_context *cmd,
 		return;
 
 check:
+	if (!hints) {
+		*vgname = name;
+		return;
+	}
+
 	/*
 	 * Only use this vgname hint if there are hints that contain this
 	 * vgname.  This might happen if we aren't able to properly extract the
@@ -1316,7 +1394,7 @@ int get_hints(struct cmd_context *cmd, struct dm_list *hints_out, int *newhints,
 	}
 
 	/*
-	 * couln't read file for some reason, not normal, just skip using hints
+	 * couldn't read file for some reason, not normal, just skip using hints
 	 */
 	if (!_read_hint_file(cmd, &hints_list, &needs_refresh)) {
 		log_debug("get_hints: read fail");
@@ -1335,13 +1413,22 @@ int get_hints(struct cmd_context *cmd, struct dm_list *hints_out, int *newhints,
 		log_debug("get_hints: needs refresh");
 		free_hints(&hints_list);
 
+		/*
+		 * This is not related to hints, and is probably unnecessary,
+		 * but it could possibly help.  When hints become invalid it's
+		 * usually becaues devs on the system have changed, and that
+		 * also means that a missing devices file entry might be found
+		 * by searching devices again.  (the searched_devnames
+		 * mechanism should eventually be replaced)
+		 */
+		unlink_searched_devnames(cmd);
+
 		if (!_lock_hints(cmd, LOCK_EX, NONBLOCK))
 			return 0;
 
 		/* create new hints after scan */
 		*newhints = NEWHINTS_REFRESH;
 		return 0;
-
 	}
 	
 	/*
@@ -1369,12 +1456,12 @@ int get_hints(struct cmd_context *cmd, struct dm_list *hints_out, int *newhints,
 	 * us which devs are PVs. We might want to enable this optimization
 	 * separately.)
 	 */
-	_get_single_vgname_cmd_arg(cmd, &hints_list, &vgname);
+	get_single_vgname_cmd_arg(cmd, &hints_list, &vgname);
 
 	_apply_hints(cmd, &hints_list, vgname, devs_in, devs_out);
 
-	log_debug("get_hints: applied using %d other %d",
-		  dm_list_size(devs_out), dm_list_size(devs_in));
+	log_debug("get_hints: applied using %d other %d vgname %s",
+		  dm_list_size(devs_out), dm_list_size(devs_in), vgname ?: "");
 
 	dm_list_splice(hints_out, &hints_list);
 
