@@ -26,6 +26,9 @@ g_tmo = 0
 # Prefix on created objects to enable easier clean-up
 g_prefix = os.getenv('PREFIX', '')
 
+# Check dev dir prefix for test suite  (LVM_TEST_DEVDIR
+dm_dev_dir = os.getenv('DM_DEV_DIR', '/dev')
+
 # Use the session bus instead of the system bus
 use_session = os.getenv('LVM_DBUSD_USE_SESSION', False)
 
@@ -62,7 +65,7 @@ def vg_n(prefix=None):
 	name = rs(8, '_vg')
 	if prefix:
 		name = prefix + name
-	return name
+	return g_prefix + name
 
 
 def lv_n(suffix=None):
@@ -70,7 +73,7 @@ def lv_n(suffix=None):
 		s = '_lv'
 	else:
 		s = suffix
-	return g_prefix + rs(8, s)
+	return rs(8, s)
 
 
 def _is_testsuite_pv(pv_name):
@@ -1811,7 +1814,7 @@ class TestDbusService(unittest.TestCase):
 			pv_object_path, vg.Vg.Pvs, "Expecting PV object path in Vg.Pvs")
 
 		lv = self._create_lv(
-			vg=vg.Vg, size=vg.Vg.FreeBytes, suffix="_pv")
+			vg=vg.Vg, size=vg.Vg.FreeBytes, suffix="_pv0")
 		device_path = '/dev/%s/%s' % (vg.Vg.Name, lv.LvCommon.Name)
 		new_pv_object_path = self._pv_create(device_path)
 
@@ -1837,8 +1840,10 @@ class TestDbusService(unittest.TestCase):
 		# by editing lvm.conf with "devices/scan_lvs = 1"  As testing
 		# typically utilizes loopback, this test is skipped in
 		# those environments.
-		pv_object_path = self.objs[PV_INT][0].object_path
 
+		if dm_dev_dir != '/dev':
+			raise unittest.SkipTest('test not running in real /dev')
+		pv_object_path = self.objs[PV_INT][0].object_path
 		if not self.objs[PV_INT][0].Pv.Name.startswith("/dev"):
 			raise unittest.SkipTest('test not running in /dev')
 
@@ -1856,6 +1861,9 @@ class TestDbusService(unittest.TestCase):
 		pv = self.objs[PV_INT][0]
 		pv_device_path = pv.Pv.Name
 
+		if dm_dev_dir != '/dev':
+			raise unittest.SkipTest('test not running in real /dev')
+
 		if not pv_device_path.startswith("/dev"):
 			raise unittest.SkipTest('test not running in /dev')
 
@@ -1867,8 +1875,9 @@ class TestDbusService(unittest.TestCase):
 
 		# Lets locate a symlink for it
 		devices = glob('/dev/disk/*/*')
+		rp_pv_device_path = os.path.realpath(pv_device_path)
 		for d in devices:
-			if pv_device_path == os.path.realpath(d):
+			if rp_pv_device_path == os.path.realpath(d):
 				symlink = d
 				break
 
@@ -1878,14 +1887,16 @@ class TestDbusService(unittest.TestCase):
 		rc = self._lookup(symlink)
 		self.assertEqual(rc, '/')
 
-		pv_object_path = self._pv_create(symlink)
+		### pv_object_path = self._pv_create(symlink)
+		### Test is limited by filter rules and must use  /dev/mapper/LVMTEST path
+		pv_object_path = self._pv_create(pv_device_path)
+
 		self.assertNotEqual(pv_object_path, '/')
 
 		pv_proxy = ClientProxy(self.bus, pv_object_path, interfaces=(PV_INT, ))
 		self.assertEqual(pv_proxy.Pv.Name, pv_device_path)
 
 		# Lets check symlink lookup
-		self.assertEqual(pv_object_path, self._lookup(symlink))
 		self.assertEqual(pv_object_path, self._lookup(pv_device_path))
 
 	def _create_vdo_pool_and_lv(self, vg_prefix="vdo_"):
